package org.relaxng.datatype;

/**
 * Datatype object.
 * 
 * Every datatype object has several functionalities. Particularly,
 * 
 * <ol>
 *  <li> ability to identifies a class of character sequences. This is
 *       done through allows method.
 * 
 *  <li> ability to produce "value object" from a character sequence and
 *		 a context information.
 * 
 *  <li> ability to test equality of two value objects.
 * </ol>
 * 
 * This interface also defines optional diagnose method, which provides
 * human-friendly error diagnosis.
 * 
 * @author <a href="mailto:kohsuke.kawaguchi@eng.sun.com">Kohsuke KAWAGUCHI</a>
 */
public interface DataType {
	
	/**
	 * checks if 'literal' matchs this datatype under the current context.
	 * 
	 * @param literal
	 *		the lexical representation to be verified
	 * @param context
	 *		context information that may be
	 *		necessary to verify the given literal.
	 * 
	 * @return
	 *		true if 'literal' is a member of this datatype;
	 *		false if it's not a member of this datatype.
	 */
	boolean allows( String literal, ValidationContext context );
	
	/**
	 * diagnoses the reason of error.
	 * 
	 * Application can call this method to provide detailed error message to users.
	 * This method is kept separate from verify method to achieve higher performance
	 * during normal validation. This method is optional and may not be implemented
	 * by some of datatypes.
	 * 
	 * <p>
	 * This method is optional and implementations are free to throw
	 * UnsupportedOperationException to indicate that this method is
	 * not implemented.
	 * 
	 * @exception UnsupportedOperationException
	 *		if diagnosis is not supported by the callee.
	 * @exception DataTypeException
	 *		If the callee supports the diagnosis and the given literal is invalid,
	 *		then this exception that contains diagnosis information is thrown.
	 */
	void check( String literal, ValidationContext context )
		throws UnsupportedOperationException, DataTypeException;
	
	/**
	 * creates an instance of the streaming validator for this type.
	 * 
	 * By using streaming validators instead of the accepts method,
	 * the caller can avoid keeping the entire string into the memory,
	 * which is sometimes quite big.
	 * 
	 * <p>
	 * This method is optional and implementations are free to throw
	 * UnsupportedOperationException to indicate that this method is
	 * not implemented.
	 * 
	 * @param context
	 *		context information that may be
	 *		necessary to verify the given literal.
	 *		The callee may keep a reference to this context object
	 *		while the returned streaming validator is being used.
	 * 
	 * @exception UnsupportedOperationException
	 *		if streaming validation is not supported by the callee.
	 */
	DataTypeStreamingValidator createStreamingValidator( ValidationContext context )
		throws UnsupportedOperationException;
	
	/**
	 * gets type name suitable for display
	 * (something like "string","dateTim", and so on.)
	 * 
	 * the callee cannot return null.
	 */
	String displayName();
	
	/**
	 * converts lexcial value and the current context to the corresponding
	 * value object.
	 * 
	 * <p>
	 * The caller cannot generally assume that the value object is a meaningful
	 * Java object. For example, the caller cannot expect this method to return
	 * java.lang.Number type for the "integer" type of XML Schema Part2.
	 * 
	 * <p>
	 * The returned value object should be used solely for the sameValue method.
	 * 
	 * @return	null
	 *		when the given lexical value is not a valid lexical value for this type.
	 */
	Object createValue( String literal, ValidationContext context );
	
	/**
	 * tests the equality of two value objects which belongs to this type.
	 * 
	 * The bahavior is undefined if objects not belonged to this type is passed.
	 * It is the caller's responsibility to ensure that objects
	 * belong to this type.
	 * 
	 * @return
	 *		true if two value objects are considered equal;
	 *		false if otherwise.
	 */
	boolean sameValue( Object value1, Object value2 );
}
